#pragma once
#if !(defined(linux) || defined(__linux) || defined(__linux__))
#include <Windows.h>
#include <stdio.h>
#else
#include <stdint.h>
#include <iostream>

typedef uint32_t DWORD;
typedef int32_t LONG;
typedef uint8_t BYTE;

#define PACKED  __attribute__((packed))

/* bitmap file format according to msdn */

typedef struct PACKED {
    uint16_t bfType;
    DWORD bfSize;
    uint16_t bfReserved1, bfReserved2;
    DWORD bfOffBits;
} BITMAPFILEHEADER;

typedef struct PACKED {
    DWORD biSize;
    int32_t biWidth, biHeight;
    uint16_t biPlanes, biBitCount;
    DWORD biCompression;
    DWORD biSizeImage;
    int32_t biXPelsPerMeter, biYPelsPerMeter;
    DWORD biClrUsed, biClrImportant;
} BITMAPINFOHEADER;

#define BI_RGB 0L

typedef struct PACKED {
    uint8_t rgbBlue, rgbGreen, rgbRed, rgbReserved;
} RGBQUAD;
#endif

class BmpHelper
{
public:
    virtual void SaveToBmpFile(const char* fname, LONG width, LONG height, const unsigned char *buffer);

protected:
    BmpHelper() {};
    virtual ~BmpHelper() {};
    virtual void InitHeaders();
    virtual void InitHeadersExt() {};
    virtual void WriteBmpHeader(FILE *f, LONG width, LONG height);
    virtual void WriteBmpHeaderExt(FILE *f, LONG width, LONG height) {};
    virtual size_t GetBmpHeaderExtSize() const { return 0; }
    virtual int GetBytesPerPixel() const = 0;
    LONG sHeader_;
    BITMAPFILEHEADER fHeader_;
    BITMAPINFOHEADER iHeader_;
    LONG GetBmpRowSize(LONG width) const;

private:
    BmpHelper(const BmpHelper &);
    BmpHelper &operator=(const BmpHelper &);
};

class BmpHelper8 : public BmpHelper
{
public:
    BmpHelper8();
    ~BmpHelper8() {}

private:
    virtual void InitHeadersExt();
    virtual void WriteBmpHeaderExt(FILE *f, LONG width, LONG height);
    virtual size_t GetBmpHeaderExtSize() const;
    virtual int GetBytesPerPixel() const { return 1; }
    RGBQUAD p8bpp[256]; // greyscale.

    BmpHelper8(const BmpHelper8 &);
    BmpHelper8 &operator=(const BmpHelper8 &);
};

class BmpHelper24 : public BmpHelper
{
public:
    BmpHelper24();
    ~BmpHelper24() {}

private:
    virtual int GetBytesPerPixel() const { return 3; }

    BmpHelper24(const BmpHelper24 &);
    BmpHelper24 &operator=(const BmpHelper24 &);
};
