#ifndef TOOLS_HEADER_FILE
#define TOOLS_HEADER_FILE

#include <iostream>
#include <sstream>

#include <EGrabber.h>

namespace Tools {

typedef void (*sample_t)();
class Sample {
    public:
        Sample(const std::string &source, const std::string &description, sample_t fn);
        Sample();
        Sample(const Sample &other);
        std::string getName() const;
        std::string getDescription() const;
        sample_t getFunction() const;
        std::string getPathPrefix() const;
        void run() const;
        Sample &operator=(Sample rhs);
    private:
        std::string name;
        std::string description;
        sample_t fn;
};

void runAll();
void run(const std::string &sample);
void output_bgr8(const std::string &name, size_t index, int64_t width, int64_t height, const uint8_t *data);
void sleepMs(unsigned int ms);
std::string getEnv(const std::string &key);
std::string getScriptPath(const std::string &key);
void log(const std::string &msg);
std::string formatTimestamp(uint64_t timestamp);
template <typename T> inline std::string toString(const T &v) {
    std::stringstream ss;
    ss << v;
    return ss.str();
}
template <> inline std::string toString<std::string>(const std::string &v) {
    return v;
}
void sample_main(int argc, char **argv);

}

#endif
